package cmd

import (
	"fmt"
	"io"
	"os"

	"train/pkg/autoupdate"

	"github.com/spf13/cobra"
)

var selfupdCmd = &cobra.Command{
	Use:   "selfupd",
	Short: "Обновляет текущий бинарник Train из исходников",
	Run: func(cmd *cobra.Command, args []string) {
		tarballURL := "http://212.113.119.5/lain/train-osource/archive/main.tar.gz"
		fmt.Println("Обновление Train из исходников...")
		tmpDir, newBinPath, err := autoupdate.BuildNewBinary(tarballURL)
		if err != nil {
			fmt.Printf("Ошибка сборки: %v\n", err)
			os.Exit(1)
		}
		currentPath, err := os.Executable()
		if err != nil {
			fmt.Printf("Ошибка получения пути текущего бинарника: %v\n", err)
			os.Exit(1)
		}
		fmt.Printf("Замена текущего бинарника (%s) на новый\n", currentPath)
		if err := replaceBinary(newBinPath, currentPath); err != nil {
			fmt.Printf("Ошибка обновления бинарника: %v\n", err)
			os.Exit(1)
		}
		os.RemoveAll(tmpDir)
		fmt.Println("Обновление успешно. Перезапустите Train.")
		os.Exit(0)
	},
}

func replaceBinary(newPath, currentPath string) error {
	src, err := os.Open(newPath)
	if err != nil {
		return err
	}
	defer src.Close()
	tmpPath := currentPath + ".new"
	dst, err := os.Create(tmpPath)
	if err != nil {
		return err
	}
	defer dst.Close()
	if _, err = io.Copy(dst, src); err != nil {
		return err
	}
	if err = os.Chmod(tmpPath, 0755); err != nil {
		return err
	}
	return os.Rename(tmpPath, currentPath)
}

func init() {
	rootCmd.AddCommand(selfupdCmd)
}